@@ -1,6 +1,6 @@
 package Farabi::Editor;
 {
-  $Farabi::Editor::VERSION = '0.34';
+  $Farabi::Editor::VERSION = '0.35';
 }
 
 # ABSTRACT: Controller
@@ -50,7 +50,8 @@ sub perl_critic {
 sub _capture_cmd_output {
 	my $self   = shift;
 	my $cmd    = shift;
-	my $source = shift->{source};
+	my $opts   = shift;
+	my $source = shift;
 
 	# Check source parameter
 	unless ( defined $source ) {
@@ -64,7 +65,7 @@ sub _capture_cmd_output {
 	close $tmp;
 
 	my ( $stdout, $stderr, $exit ) = capture {
-		system( $cmd, $tmp->filename );
+		system( $cmd, @$opts, $tmp->filename );
 	};
 	my $result = {
 		stdout => $stdout,
@@ -76,19 +77,21 @@ sub _capture_cmd_output {
 }
 
 sub run_perl {
-	$_[0]->_capture_cmd_output( $^X, $_[1] );
-}
-
-sub run_niecza {
-	$_[0]->_capture_cmd_output( 'Niecza.exe', $_[1] );
+	my $self   = shift;
+	my $source = shift->{source};
+	$self->_capture_cmd_output( $^X, [], $source );
 }
 
 sub run_rakudo {
-	$_[0]->_capture_cmd_output( 'perl6', $_[1] );
+	my $self   = shift;
+	my $source = shift->{source};
+	$self->_capture_cmd_output( 'perl6', [], $source );
 }
 
 sub run_parrot {
-	$_[0]->_capture_cmd_output( 'parrot', $_[1] );
+	my $self   = shift;
+	my $source = shift->{source};
+	$self->_capture_cmd_output( 'parrot', [], $source );
 }
 
 # Taken from Padre::Plugin::PerlTidy
@@ -992,6 +995,30 @@ sub find_plugins {
 	return \@plugins;
 }
 
+# Syntax check the provided source string
+sub syntax_check {
+	my $self   = shift;
+	my $source = shift->{source};
+
+	my $result = $self->_capture_cmd_output( "$^X", ["-c"], $source );
+
+	require Parse::ErrorString::Perl;
+	my $parser = Parse::ErrorString::Perl->new;
+	my @errors = $parser->parse_string( $result->{stderr} );
+
+	my @problems;
+	foreach my $error (@errors) {
+		push @problems,
+		  {
+			message => $error->message,
+			file    => $error->file,
+			line    => $error->line,
+		  };
+	}
+
+	return \@problems;
+}
+
 # The default root handler
 sub default {
 	my $self = shift;
@@ -1011,7 +1038,7 @@ sub websocket {
 	require Mojo::JSON;
 	my $json = Mojo::JSON->new;
 
-	# Disable inactivity timeout 
+	# Disable inactivity timeout
 	Mojo::IOLoop->stream( $self->tx->connection )->timeout(0);
 
 	# Wait for a WebSocket message
@@ -1026,7 +1053,6 @@ sub websocket {
 				'find-file'                => 1,
 				'open-file'                => 1,
 				'run-perl'                 => 1,
-				'run-niecza'               => 1,
 				'run-rakudo'               => 1,
 				'run-parrot'               => 1,
 				'help_search'              => 1,
@@ -1035,6 +1061,7 @@ sub websocket {
 				'pod2html'                 => 1,
 				'pod-check'                => 1,
 				'save-file'                => 1,
+				'syntax-check'             => 1,
 				'find-duplicate-perl-code' => 1,
 				'find-plugins'             => 1,
 				'repl-eval'                => 1,
@@ -1047,6 +1074,9 @@ sub websocket {
 				my $o = $self->$action( $result->{params} ) or return;
 				$ws->send( $json->encode($o) );
 			}
+			else {
+				$self->app->log->warn("'$action' not found!");
+			}
 
 		}
 	);
@@ -1064,7 +1094,7 @@ Farabi::Editor - Controller
 
 =head1 VERSION
 
-version 0.34
+version 0.35
 
 =head1 AUTHOR
 
